/* The code used in this script to leverage the CSSDrawing library is based on
   example 22-7 in the book JavaScript: The Definitive Guide, 5th Edition.
   by David Flanagan. Copyright 2006 O'Reilly Media, Inc. (ISBN #0596101996)
   
   Be sure to include David Flanagan's CSSDrawing library (found at www.davidflanagan.com/javascript5)
   in your html.
  
   The code also uses a cross-browser split method by Steven Levithan, which also
   requires an external file (available at http://stevenlevithan.com/assets/misc/split.js).
   The cross-browser split code is licensed under the MIT license.
*/
window.onload = setup ;


var filename = "genetic_records_sample.txt";



var xhr = false ;
//var queryType = document.body.famtree.display.checked;//"FamilyTree";
var individuals = new Array() ;   // list to look for parents
var families = new Array() ;      // results of ancestor and child search
var rcdList = new Object() ;      // full list of individuals and parents
var colNames = new Array();	  // stores column headings for properties of rcdList objects
var keyIndex = new Array() ;	  // list of IDs in order they were read - used to generate lists
var numDuplicates = 0;			  // tracks how many sequences are duplicated in tree
var duplicateMarker = "*";		  // symbol on tree used to mark duplicates
var defaultNGen = 5;		
var display;

//constant upper limit for number of sequences in one year
var upperBound = 2000;

//constants for tree graphics

var box_x = 54 //keeping this quantity even avoids pixel gaps in some browsers
var box_y = 20;
var border = 2; //thickness of box borders - make sure it corresponds to css file
var line_top_offset = (box_x + border)/2;
var line_side_offset = (box_y + border)/2;
var node_x_offset;
var node_y_offset = 2*box_y;
var not_found_increase = 10;

//for popup
var popup_offset = 25;
var popup_x = 4*box_x;
var popup_y;// = 7*box_y/2;
var extra_row = 14;

//for duplicate tagging
var tag_x = 10
var tag_y = 10;

//dynamically defined graphics quantities
var dim_x;
var dim_y;
var start_x;
var start_y;
var initial_node_x_offset;
var childspread;


var promptString = "Mouse over the elements in the tree to see their individual derivations and genetic texts.  Click on an element to see its family tree." ;
var duplicatePrompt = "Sequences that appear more than once on this tree are tagged with asterisks.  All duplicates of the same sequence are tagged with the same number of asterisks." ;

function setup() {
   makeRequest(filename) ;
   showForTree();
   document.famtree.onsubmit = function() { return false } ;  // don't resubmit page
   document.getElementById("btnNewTree").onclick = showForTree;
   document.getElementById("btnFamilyList").onclick = showForList;
   document.getElementById("btnSearch").onclick = showForSearch;
   document.getElementById("submitBtn").onclick = submitQuery;
}

//thanks to Antonio Lupetti on woorkup.com for some code ideas here
function showForTree()
{
	document.getElementById("famForm").setAttribute("class", "showForm");
	document.getElementById("treeTxt").setAttribute("class", "showForm");
	document.getElementById("listTxt1").setAttribute("class", "hiddenForm");
	document.getElementById("listTxt2").setAttribute("class", "hiddenForm");
	document.getElementById("famSearchForm").setAttribute("class", "hiddenForm");
	document.getElementById("ownerForm").setAttribute("class", "hiddenForm");
	document.getElementById("searchForm").setAttribute("class", "hiddenForm");
}

function showForList()
{
	document.getElementById("famForm").setAttribute("class", "showForm");
	document.getElementById("treeTxt").setAttribute("class", "hiddenForm");
	document.getElementById("listTxt1").setAttribute("class", "showForm");
	document.getElementById("listTxt2").setAttribute("class", "showForm");
	document.getElementById("famSearchForm").setAttribute("class", "hiddenForm");
	document.getElementById("ownerForm").setAttribute("class", "showForm");
	document.getElementById("searchForm").setAttribute("class", "hiddenForm");
}

function showForSearch()
{
	document.getElementById("famForm").setAttribute("class", "hiddenForm");
	document.getElementById("treeTxt").setAttribute("class", "hiddenForm");
	document.getElementById("listTxt1").setAttribute("class", "hiddenForm");
	document.getElementById("listTxt2").setAttribute("class", "hiddenForm");
	document.getElementById("famSearchForm").setAttribute("class", "showForm");
	document.getElementById("ownerForm").setAttribute("class", "showForm");
	document.getElementById("searchForm").setAttribute("class", "showForm");
}

function submitQuery(){
	var i;
	for(i = 0; i < document.famtree.display.length; i++){
		if(document.famtree.display[i].checked) break;
	}
	var queryType = document.famtree.display[i].value;
	if (queryType == "Family Tree"){
		newTree();
	} else if (queryType == "Family List") {
		newList();
	} else if (queryType == "Search") {
		newSearch();
	}
}

function rolloverInit() {
    var nodes = document.getElementsByTagName("div");
    for (var i = 0; i < nodes.length ; i++) {
        if(nodes[i].id == "TreeNode") setupRollover(nodes[i]);
      } 
    var cells = document.getElementsByTagName("td");
    for (i = 0; i < cells.length ; i++) {
        if(cells[i].id == "TableNode") setupClick(cells[i]);
      } 
}

function setupClick(thisCell)
{
	thisCell.onclick = redrawTree;
}

function setupRollover(thisNode)
{
	thisNode.onmouseover = rollOver;
	thisNode.onmouseout = rollOff;
	thisNode.onclick = redrawTree;	
}

function rollOver(e) {

//These two lines written by Peter-Paul Koch: 	http://www.quirksmode.org/js/events_access.html
	if (!e) var e = window.event;
	// e gives access to the event in all browsers
//end Peter-Paul Koch's code
   var familyName = this.innerHTML ;
   this.id = "TreeNodeHover";
   
   var showFam = familyName;
   var showText = "NOT FOUND";
   var showDeriv = "NOT FOUND"; 

if(rcdList[familyName])
   {
   		showDeriv = rcdList[familyName].mother+" x "+rcdList[familyName].father;
		showText = rcdList[familyName].text;
   }	
	//Display mouseover-popup viewbox
	
	var x_coord = mouseX(e);
	var y_coord = mouseY(e); 
	shortText = familyName+" &nbsp "+showDeriv;
	popup = new CSSDrawing(x_coord, y_coord, 2*box_x, 2* box_y, "currentPopup");
	popup_y = setHeight(shortText, showText/*showGenTxt*/);
	popup.box(popup_offset,popup_offset, popup_x, popup_y, shortText+"<br>"+showText /*showGenTxt*/, "PopupViewer");
	popup.insert(document.getElementById("popupBox"));
	
    return false ;
}

function setHeight(shortText, genText)
{
	var nExtraRows = 1 + (shortText.length / 32) + (genText.length / 32);//1 additional line needed for every set of 32 additional characters
	return(box_y + nExtraRows*extra_row);
}

/*This code written by Stephen Chapman, found on About.com: http://javascript.about.com/library/blmousepos.htm */

function mouseX(evt) {
if (evt.pageX) return evt.pageX;
else if (evt.clientX)
   return evt.clientX + (document.documentElement.scrollLeft ?
   document.documentElement.scrollLeft :
   document.body.scrollLeft);
else return null;
}

function mouseY(evt) {
if (evt.pageY) return evt.pageY;
else if (evt.clientY)
   return evt.clientY + (document.documentElement.scrollTop ?
   document.documentElement.scrollTop :
   document.body.scrollTop);
else return null;
}

/*end Stephen Chapman's code*/

function rollOff(){
	this.id = "TreeNode"; //restores z-coordinate to 0
    var oldPopup = document.getElementById("popupBox");
    while (oldPopup.childNodes[0]){
   	  oldPopup.removeChild(oldPopup.childNodes[0]);
   	}
    return false;
}
function redrawTree(){
	var familyName = this.innerHTML ;
	document.getElementById("famYearNum").value = familyName;
	document.getElementById("btnNewTree").checked = true;
	document.getElementById("btnFamilyList").checked = false;
	document.getElementById("btnSearch").checked = false;
	if (document.getElementById("generations").value > defaultNGen) document.getElementById("generations").value = defaultNGen;
	showForTree();
	newTree();
	return false;
}

function newTree() {   


    // load individual
    var individual = document.getElementById("famYearNum").value.toUpperCase().replace(/\s+/g, "") ;


    //Validate input
    if (!rcdList[individual] || (individual == ""))
    {
	alert("Query individual not found.");
	document.getElementById("famYearNum").select();
	return;
    } 
    if (getNGen() < 1)
    {
	alert("Invalid number of generations.");
	document.getElementById("generations").select();
	return;
    } 

    
    reset(true) ;    // clear last tree
    var tablebody ;
    var lvlPref ;
    
    
    //Fields are sequence name, classname, maternal/paternal, x coordinate, y coordinate, index of child node if applicable
    individuals[0] = [individual, "IndRel", "IndRel" , start_x, start_y, -1] ;

    // build table starting with recent parents - 5 generations
    var j = 0;
    var nGen = document.getElementById("generations").value ;
    if (nGen == "") nGen = defaultNGen;
    for (var i = 0 ; i <= getNGen() ; i++ ) {             
       if (i == 0 ) {
          lvlPref = "F" ;
       }
       else {
          lvlPref = "P" ;
       }
		

       /* add all necessary records to families array using individuals array to find parents then 
       putting parents into individuals array */
       addParents() ;
       for (; j < families.length ; j++ ) {
          drawNode(families[j]);
          addRow(i, families[j], lvlPref) ;

       }
       
       node_x_offset /= 2;
       // end of each generation, border 4 px using class on TR element
       tablebody = document.getElementsByTagName("TBODY")[0] ;
       tablebody.childNodes[tablebody.childNodes.length-1].className = "LastRow" ;
    }
    addChildren() ;
    drawChildren(families.length - j) ; 
    for ( ; j < families.length ; j++ ) {    
       addRow(1, families[j], "F") ;
    }
       // end of each generation, border 4 px using class on TR element
       tablebody = document.getElementsByTagName("TBODY")[0] ;
       tablebody.childNodes[tablebody.childNodes.length-1].className = "LastRow" ;
       
//WORK IN PROGRESS
       if(document.getElementById("generations") == "") document.getElementBy
       display.insert(document.getElementById("familytree"));
       rolloverInit();
       if(document.getElementById("generations").value == "") document.getElementById("generations").value = 5;
       document.getElementById("famYearNum").select();
}

function addChildren() {

    var individual = document.getElementById("famYearNum").value.toUpperCase().replace(/\s+/g, "") ;
	if(individual == "") return;

    for (var indv in rcdList) {
        // used as female parent
        if ( rcdList[indv].mother == individual ) {
        	var family = {}
			family.name = indv;
			family.side = "ChildRel";
			family.mother = rcdList[indv].mother;
			family.father = rcdList[indv].father;
			family.text = rcdList[indv].text;
			family.owner = rcdList[indv].owner;
			family.index = rcdList[indv].index;
			//x and y will be set later
			family.childIndex = -1;
        	//self-cross
            if ( rcdList[indv].father == individual ) {
              family.className = "SelfXRel";
              families[families.length] = family;
            }
            else {
              family.className = "FemaleRel";
              families[families.length] = family;
            } 
        }
        // used as male parent only
        else if( rcdList[indv].father == individual ){
           var family = {};
           family.name = indv;
		   family.className = "MaleRel";
		   family.side = "ChildRel";
		   family.mother = rcdList[indv].mother;
		   family.father = rcdList[indv].father;
		   family.text = rcdList[indv].text;
		   family.owner = rcdList[indv].owner;
		   family.index = rcdList[indv].index;
		   //x and y will be set later
		   family.childIndex = -1;
           families[families.length] = family;
        }
    }
}

function drawChildren(numChildren) {
	if (numChildren == 0) return;
	var child_offset;
	var child_compress = 1;
 	if (numChildren < 8) child_compress = 2 ;
 	childspread = childspread / child_compress ;
	if(numChildren > 1) child_offset = childspread/(numChildren-1);
	var familiesOffset = families.length - numChildren;
	var firstChild_x = start_x - childspread/2;
	var child_x = start_x; //initialize to value for case numChildren ==1;
	var child_y = start_y+2*node_y_offset;
	//draw each child and a vertical line above its box
	for( var i = 0; i < numChildren; i ++)
	{
		if(numChildren > 1) child_x = firstChild_x+i*child_offset;
		
		//set node location in appropriate fields in families array
		families[familiesOffset + i].x = child_x;
		families[familiesOffset + i].y = child_y;	
		
		addBox(families[familiesOffset + i]);
		addChildLine(child_x, child_y);
	}
	//draw two lines to link all the children to their parent
	if(numChildren > 1)display.horizontal(firstChild_x + line_top_offset, child_y-node_y_offset+line_side_offset, childspread, "boldline");
	display.vertical(start_x + line_top_offset, start_y+2*(line_side_offset),node_y_offset-line_side_offset, "boldline");
 	childspread = childspread * child_compress ;
}

/*Goes through list of individuals, finding and storing each element's parents and putting the 
  individual into the larger families array as a family object.  When every element in individuals 
  has been processed, the parents array becomes the new individuals array. */
function addParents() {

  var data ;
  var parents = new Array() ;

  if (individuals.length > 500) {
    alert("Over max of 500: " + individuals.length) ;
    individuals = new Array() ;    
  }

  for (var j = 0 ; j < individuals.length ; j++ ) {
     var family = {}
	 family.name = individuals[j][0];
	 family.x = individuals[j][3];
	 family.y = individuals[j][4];
	 family.childIndex = individuals[j][5];
	 family.isDuplicate = false;
     if (rcdList[family.name] && (family.name != "")) { 
       	//This code checks to see if sequence is a duplicate
       	for(var i = 0; i<families.length; i++) 
        {
        	if ( (individuals[j][0] == families[i].name) && (families[i].className != "NotFoundRel") )
        	{
        		family.isDuplicate = true;
        		//Check if this sequence has already been assigned a code
        		if(families[i].isDuplicate) 
        		{
        			family.duplicateTag = families[i].duplicateTag;
        		}
        		else
        		{
        			families[i].isDuplicate = true;
        			numDuplicates++;
        			families[i].duplicateTag = "";
        			for(var k = 0; k < numDuplicates; k++) families[i].duplicateTag += duplicateMarker;
        			tagDuplicate(families[i]); //this node might be drawn already, need to add tag
        			family.duplicateTag = families[i].duplicateTag;
        		}
        	}
        }
        if ( (!family.isDuplicate) && (rcdList[family.name].mother !== rcdList[family.name].father) )/*two different parents*/ {
           parents[parents.length] = [rcdList[family.name].mother, "FemaleRel", "FemaleRel", 
           		individuals[j][3] - node_x_offset/2 , individuals[j][4] - node_y_offset, families.length] ;   // female
           parents[parents.length] = [rcdList[family.name].father, "MaleRel", "MaleRel",
           		individuals[j][3] + node_x_offset/2 , individuals[j][4] - node_y_offset, families.length ] ;   // male
           if (individuals[j][1] != "IndRel") {
              parents[parents.length - 2][2] = individuals[j][2] ;   // keep track of maternal/paternal
              parents[parents.length - 1][2] = individuals[j][2] ;   // keep track of maternal/paternal
           }        
        }
        else if(!family.isDuplicate){
           parents[parents.length] = [rcdList[family.name].mother, "SelfXRel", "SelfXRel", 
           	individuals[j][3] - node_x_offset/2 , individuals[j][4] - node_y_offset, families.length ] ;   // self cross
           if (individuals[j][1] !== "IndRel") {
              parents[parents.length - 1][2] = individuals[j][2] ;   // keep track of maternal/paternal
           }
        }    

		family.className = individuals[j][1];
		family.mother = rcdList[family.name].mother;
		family.father = rcdList[family.name].father;
		family.text = rcdList[family.name].text;
		family.owner = rcdList[family.name].owner;
		family.index = rcdList[family.name].index;
     }
     else {
		if(family.name == "") family.name = "Not Found";
		family.className = "NotFoundRel";
		family.mother = "Not Found";
		family.father = "Not Found";		
		family.text = "Not Found";
     }   
          //keep track of maternal/paternal
	 if(family.childIndex < 0)
	 {
	 	family.side = "IndRel";
	 }
	 else if(family.childIndex == 0)
	 {
	 	family.side = family.className;
	 }
	 else
	 {
	 	family.side = families[family.childIndex].side;
	 }
	 families[families.length] = family;
  }
  
  individuals = parents ;
  
  return false ;

}

//takes an entry from the families array and draws a node representing it on the tree with appropriate connecting lines
function drawNode(entry){
	
	if(entry.childIndex >= 0){  //check if this entry is an ancestor or not - child index will be greater than or equal to zero if an ancestor
		var child_x = families[entry.childIndex].x;
		var child_y = families[entry.childIndex].y;
    	addChildLine(child_x, child_y);
    	var type = "left";
    	
    	//Find out if this ancestor is a male parent
    	var childsFather = rcdList[families[entry.childIndex].name].father;
    	var childsMother = rcdList[families[entry.childIndex].name].mother;   
    	var isMale = (entry.className == "MaleRel") || //might also be a NotFound relative acting as male
    		((entry.className == "NotFoundRel" )&&( entry.name == childsFather)&&(entry.name != childsMother)); 
    	if(isMale) type = "right";
    	addParentLine(child_x, child_y, type);
    }
    addBox(entry);
    if(entry.isDuplicate) tagDuplicate(entry);
}


function tagDuplicate(entry)
{
	//create and display tag marking this sequence as a duplicate

	//vertically right-biased version
	var x_coord = entry.x + box_x/2 + 3*border/2 ;
	var y_coord = entry.y - node_y_offset/3;
	display.box(x_coord, y_coord,tag_x, tag_y, entry.duplicateTag);	
	addChildLine(entry.x, entry.y);	
}

//Takes a family object and draws the box representing this node
function addBox(entry){
	var x_coord = entry.x;	
	var y_coord = entry.y;
	var name = entry.name;
	var type = entry.className;
	var height = box_y;
	if(type == "NotFoundRel") height += not_found_increase;
	display.box(x_coord, y_coord, box_x, height, name, type, "TreeNode"); //"<p valign=middle>"+name+"</p>", "boxstyle");
}

//Adds vertical line above child node, part of connection to parent(s)
function addChildLine(x, y){
	display.vertical(x+line_top_offset, y-(node_y_offset-line_side_offset),node_y_offset-line_side_offset, "boldline");
}

function addParentLine(child_x, child_y, type){
	var xBegin = child_x + line_top_offset;
	if(type == "left"){
		xBegin -= (node_x_offset - line_top_offset);
    }
    if(node_x_offset - line_top_offset <= 0) return;
	display.horizontal(xBegin, child_y - (node_y_offset - line_side_offset), node_x_offset - line_top_offset, "boldline");
}


function newList() {   

    // load individual
    var individual = document.getElementById("famYearNum").value.toUpperCase().replace(/\s+/g, "") ;

    //Validate input
    if (!rcdList[individual] || (individual == ""))
    {
	alert("Query individual not found.");
	document.getElementById("famYearNum").select();
	return;
    } 
    if (getNGen() < 1)
    {
	alert("Invalid number of results.");
	document.getElementById("generations").select();
	return;
    } 

    reset(false) ;    // clear last tree
    var results = 0;
    var indvIdx = -1;
    if(rcdList[individual]) indvIdx = rcdList[individual].index;
    var filter = document.getElementById("owner").value;
    var nGen = document.getElementById("generations").value;
    if(nGen == "") nGen = defaultNGen;
    for(i = indvIdx; i < indvIdx + getNGen(); i++ ) 
    //parseInt(document.getElementById("generations").value)
    {
	individual = keyIndex[i];
	var thisOwner = rcdList[individual].owner;
	if(!filter || (filter == thisOwner))
	{
        	    var family = {};
	            family.name = individual;
				family.className = "IndRel";
				family.side = "IndRel";
				family.mother = rcdList[individual].mother;
				family.father = rcdList[individual].father;
				family.text = rcdList[individual].text;
				family.owner = rcdList[individual].owner;
				family.index = rcdList[individual].index;
				results++;
				addRow(results, family, "") ;
	}
    }
    //put a thicker border at the end of the table
    tablebody = document.getElementsByTagName("TBODY")[0] ;
    tablebody.childNodes[tablebody.childNodes.length-1].className = "LastRow"
    
    rolloverInit();
    //put the cursor in the input box to allow the user to redraw the list quickly
    document.getElementById("famYearNum").select();
}

function newSearch () {
	reset(false);
	var individual = document.getElementById("famYearFilter").value.toUpperCase().replace(/\s+/g, "") ;
	var baseIdx = -1;
	if (rcdList[individual]) baseIdx = rcdList[individual].index;
	var filter = document.getElementById("owner").value;
	var searchTokens = document.getElementById("keywords").value.split(/\s+/g);
	var searchType = "or";
	if(document.getElementById("btnAnd").checked) searchType = "and";
	//for (var indv in rcdList) 
	for (var i = 0; i < keyIndex.length; i++)
	{
		//if (rcdList[indv].index >= baseIdx) 
		if(i >= baseIdx)
		{ //skip to next cycle if year is behind cutoff

			var indv = keyIndex[i];

			//var searchLine = indv;
			//for (prop in rcdList[indv]) searchLine = searchLine+" "+prop;
			var hits = 0; //used to track number of keywords matched in AND searching
			for(var j = 0; j < searchTokens.length; j++)
			{
				var word = searchTokens[j];
				//if (searchLine.indexOf(word)!= -1)
				for(prop in rcdList[indv])
				{
					 var searchLine = rcdList[indv][prop].toString();
					 if(searchLine.indexOf(word)!= -1) 
					 {
						hits++;
						break;
					 }
				}
//				if(isFound)
				
/*				if( ((rcdList[indv].text)&&(rcdList[indv].text.indexOf(word) != -1))
				|| ((rcdList[indv].mother)&&(rcdList[indv].mother.indexOf(word) != -1))
				|| ((rcdList[indv].father)&&(rcdList[indv].father.indexOf(word) != -1))
				|| ((rcdList[indv].owner)&&(rcdList[indv].owner.indexOf(word) != -1))
				)
					hits++;
*/
			}
			//filter by owner if user has entered one
			var thisOwner = rcdList[indv].owner;
			if(!filter || (filter == thisOwner))
			{
				if(((searchType == "or") && (hits > 0)) || ((searchType == "and") && (hits == searchTokens.length))) 
				{
					//families[families.length] = [indv, "IndRel", "IndRel", rcdList[indv].crossText, rcdList[indv].genText ]; //store that;  			
			            	var family = {};
		           		family.name = indv;
					family.className = "IndRel";
					family.side = "IndRel";
					family.mother = rcdList[indv].mother;
					family.father = rcdList[indv].father;
					family.text = rcdList[indv].text;
					family.owner = rcdList[indv].owner;
					family.index = rcdList[indv].index;
					families[families.length] = family;
				}
			}
		}
	}
	for(var j = 0; j <families.length; j++) addRow(j + 1, families[j], "");
	//put a thicker border at the end of the table
	tablebody = document.getElementsByTagName("TBODY")[0] ;
	tablebody.childNodes[tablebody.childNodes.length-1].className = "LastRow";	

	rolloverInit();
 	//put the cursor in the input box to allow the user to redraw the list quickly
    	document.getElementById("keywords").select();
}

function reset(isTree){
   
   //Remove old tree prompt
   var prompt = document.getElementById("prompt") ;
   if(prompt.childNodes[0]) prompt.removeChild(prompt.childNodes[0]) ;
   	
   //Remove old list if it's there
   var tablebody = document.getElementsByTagName("TBODY")[0] ;
   while (tablebody.childNodes[0]) {
      tablebody.removeChild(tablebody.childNodes[0]) ;
   }
   
   //Remove old tree
   var oldTree = document.getElementById("familytree");
   while (oldTree.childNodes[0]){
   	  oldTree.removeChild(oldTree.childNodes[0]);
   }
   
   makeTitleRow(tablebody);

   if(isTree)
   {
   		//set dimensions of tree
   		
   		dim_x = 2048; //keep this quantity some multiple of a high power of 2 to avoid pixel gaps in some browsers //YAHOO.util.Dom.getViewportWidth();
		if(Number(getNGen()) < 6) dim_x/= 2;
		start_x = dim_x/2;
		initial_node_x_offset = dim_x/2;
		childspread = 2*initial_node_x_offset - 2*(line_top_offset);
   		
   		dim_y = node_y_offset*(Number(getNGen()) + 5);
 		start_y = dim_y - 4*node_y_offset;
 		
 		
   		//Set up a new tree
   		display = new CSSDrawing(dim_x, dim_y, "currentTree");   
   
   		prompt.appendChild(document.createTextNode(promptString)) ;
	}
 
   individuals = new Array() ;
   families = new Array();
   nodeList = new Array();
   node_x_offset = initial_node_x_offset;
   numDuplicates = 0;
   
}


function addRow(i, entry, lvlPref) {

     var newRow = document.createElement("tr") ;
     
     // Level of parent
     var newCell = document.createElement("td") ;
     newCell.appendChild(document.createTextNode(lvlPref + i)) ;
     newCell.className = entry.side ;//families[j][2] //fix this ;
     newRow.appendChild(newCell);
 
     newCell = document.createElement("td") ;
     newCell.appendChild(document.createTextNode(entry.name)) ;  // individual
     newCell.className = entry.className ; 
     newCell.id = "TableNode";
     newRow.appendChild(newCell);

     if(rcdList[entry.name])
     { 
        var indv = rcdList[entry.name]
     }
     else
     {
	var indv = {};
	for(var prop in rcdList[keyIndex[1]]) indv[prop] = "";
     }
     for(var prop in indv)
     {
	     if(prop == "index") continue;
	     newCell = document.createElement("td") ;
	     newCell.appendChild(document.createTextNode(indv[prop])) ;  // individual
	     newCell.className = entry.className ; 
  	     newRow.appendChild(newCell);
     }
     
/*     // Genetic Record data
     newCell = document.createElement("td") ;
     newCell.appendChild(document.createTextNode(entry.name)) ;  // individual
     newCell.className = entry.className ; 
     newRow.appendChild(newCell);

     newCell = document.createElement("td") ;
     newCell.appendChild(document.createTextNode(entry.father)) ;  // male parent
     newCell.className = entry.className  ; 
     newRow.appendChild(newCell);

     newCell = document.createElement("td") ;
     newCell.appendChild(document.createTextNode(entry.mother)) ;  // female parent
     newCell.className = entry.className  ; 
     newRow.appendChild(newCell);

     newCell = document.createElement("td") ;
     newCell.appendChild(document.createTextNode(entry.text)) ;  // text
     newCell.className = entry.className  ; 
     newRow.appendChild(newCell);

     newCell = document.createElement("td") ;
     newCell.appendChild(document.createTextNode(entry.owner)) ;  // owner
     newCell.className = entry.className  ; 
     newRow.appendChild(newCell);
*/
     var tablebody = document.getElementsByTagName("TBODY")[0] ;
     tablebody.appendChild(newRow) ; 

}

function makeTitleRow(parent) {
   // add headings to table
   var newRow = document.createElement("tr") ;   

   var newCell = document.createElement("td") ;
   newCell.appendChild(document.createTextNode("Lvl")) ;
   newRow.appendChild(newCell);

   for(var i = 0; i < colNames.length; i++)
   {
	newCell = document.createElement("td") ;
   	newCell.appendChild(document.createTextNode(colNames[i])) ;  // individual
   	newRow.appendChild(newCell);
   }
   
 /*  // Genetic Record data
   newCell = document.createElement("td") ;
   newCell.appendChild(document.createTextNode("Individual")) ;  // individual
   newRow.appendChild(newCell);

   newCell = document.createElement("td") ;
   newCell.appendChild(document.createTextNode("Male Parent")) ;  // cross text
   newRow.appendChild(newCell);
   
   newCell = document.createElement("td") ;
   newCell.appendChild(document.createTextNode("Female Parent")) ;  // cross text
   newRow.appendChild(newCell);

   newCell = document.createElement("td") ;
   newCell.appendChild(document.createTextNode("Additional Information")) ;  // cross text
   newRow.appendChild(newCell);

   newCell = document.createElement("td") ;
   newCell.appendChild(document.createTextNode("Owner")) ;  // owner
   newRow.appendChild(newCell) ;
*/

  newRow.className = "TitleRow" ;
  parent.appendChild(newRow) ;
}

function getNGen()
{
    var nGen = document.getElementById("generations").value;
    if(nGen == "") nGen = defaultNGen;
    return parseInt(nGen);
}


function saveOwners(ownerHash) {

	var ownerForm = document.getElementById("owner");
	for (var owner in ownerHash)
	{
	   newOwner = document.createElement("option") ;
	   newOwner.value = owner;
   	   newOwner.appendChild(document.createTextNode(owner)) ;		
  	   ownerForm.appendChild(newOwner);		 	
	}
}

function getRecordList() {

  var line ;
  var key ;
  var ownerHash = new Object();
  if (xhr.readyState == 4) {
    if (xhr.status == 200) {
      var pattern = /\n/ ;
      
      // split text into lines then lines by tabs
    //   rcds = xhr.responseText ;
      var rcds = xhr.responseText.split(pattern) ;
      // loop each rcd and create object field
      
      //var validLine = /\S+/;
      for (var i = 0 ; i < rcds.length ; i++ ) {
	 if (!rcds[i]) continue;
         line = rcds[i].split(/\t/) ;
         key = line[0].toUpperCase().replace(/\s+/g, "")
	 colNames[0] = "Individual";
         rcdList[key] = {};

         rcdList[key].mother = line[1];
	 colNames[1] = "Female Parent";	 

         rcdList[key].father = line[2];
	 colNames[2] = "Male Parent";

 	 rcdList[key].text = line[3];
	 colNames[3] = "Description";

         rcdList[key].owner = line[4];
	 colNames[4] = "Owner";

	 //Additional columns for your data can be added using the same format as the columns above

	 rcdList[key].index = keyIndex.length;
	 keyIndex[keyIndex.length] = key;
	 if(ownerHash[rcdList[key].owner])
	 { 
		ownerHash[rcdList[key].owner]++;
	 }
	 else
	 {
		ownerHash[rcdList[key].owner] = 1;
	 }
      }
      saveOwners(ownerHash);
      alert("Found " + rcds.length + " family records!") ;
    }
  }
  return false ;

}


function makeRequest(url) {
   if (window.XMLHttpRequest) {
      xhr = new XMLHttpRequest() ;
    }
    else {
      alert("upgrade your browser") ;
    }
    
    if (xhr) {
      xhr.onreadystatechange = getRecordList ;   // function to run when file retrieved
      xhr.open("GET",url,true) ;
      xhr.send(null) ;
    }
    else {
      alert("Could not create XML request.") ;
    }
}